/***** histogram.c ********************************
 * Description: Compute histogram.
 * Author: Bernhard Haubold, haubold@evolbio.mpg.de
 * Date: Fri Jun  8 12:09:43 2012
 **************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <gsl/gsl_histogram.h>
#include "stringUtil.h"
#include "histogram.h"
#include "interface.h"
#include "eprintf.h"

void printLinePlot(gsl_histogram *hist){
  size_t i, n;
  double x, l, u, p;

  n = gsl_histogram_bins(hist);
  for(i=0; i<n; i++){
    gsl_histogram_get_range(hist, i, &l, &u);
    x = gsl_histogram_get(hist, i);
    p = l + (u-l) / 2.;
    printf("%g\t%g\n",p,x);
  }
}

void printStepPlot(gsl_histogram *hist){
  size_t i, n;
  double x, l, u;

  n = gsl_histogram_bins(hist);
  if(n>1){
    gsl_histogram_get_range(hist, 0, &l, &u);
    printf("%g\t0\n", l);
  }
  for(i=0; i<n; i++){
    gsl_histogram_get_range(hist, i, &l, &u);
    x = gsl_histogram_get(hist, i);
    printf("%g\t%g\n",l,x);
    printf("%g\t%g\n",u,x);
  }
  if(n>1)
    printf("%g\t0\n", u);
}

void printBoxPlot(gsl_histogram *hist){
  size_t i, n;
  double x, l, u;

  n = gsl_histogram_bins(hist);
  for(i=0; i<n; i++){
    gsl_histogram_get_range(hist, i, &l, &u);
    x = gsl_histogram_get(hist, i);
    printf("%g\t0\n", l);
    printf("%g\t%g\n",l,x);
    printf("%g\t%g\n",u,x);
  }
  printf("%g\t0\n",u);
}

void scanFile(FILE *fp, Args *args){
  double min, max, x;
  double *array;
  gsl_histogram *hist;
  size_t arraySize, n;
  int i;

  /* get input data */
  min = DBL_MAX;
  max = -DBL_MAX;
  arraySize = 1;
  array = (double *)emalloc(arraySize*sizeof(double));
  n = 0;
  while(fscanf(fp, "%lg", &x) == 1){
    if(x > max)
      max = x;
    if(x < min)
      min = x;
    array[n++] = x;
    if(n == arraySize){
      arraySize *= 2;
      array = (double *)erealloc(array,arraySize*sizeof(double));
    }
  }
  array = (double *)erealloc(array,n*sizeof(double));
  /* construct histogram */
  hist = gsl_histogram_alloc(args->b);
  if(!args->R){
    max += DBL_EPSILON;
    min -= DBL_EPSILON;
    gsl_histogram_set_ranges_uniform(hist, min, max);
  }else
    gsl_histogram_set_ranges_uniform(hist, args->R[0], args->R[1]);
  for(i=0;i<n;i++)
    gsl_histogram_increment(hist, array[i]);
  if(!args->r)
    gsl_histogram_scale(hist, 1./n);
  if(args->B)
    printBoxPlot(hist);
  else if(args->L)
    printLinePlot(hist);
  else
    printStepPlot(hist);
  gsl_histogram_free(hist);
  free(array);
}

int main(int argc, char *argv[]){
  int i;
  char *version;
  Args *args;
  FILE *fp;

  version = "0.13";
  setprogname2("histogram");
  args = getArgs(argc, argv);
  if(args->h || args->e)
    printUsage(version);
  if(args->v)
    printSplash(version);
  if(args->numInputFiles == 0){
    fp = stdin;
    scanFile(fp, args);
  }else{
    for(i=0;i<args->numInputFiles;i++){
      fp = efopen(args->inputFiles[i],"r");
      scanFile(fp, args);
      fclose(fp);
    }
  }
  freeArgs();
  free(progname());
  return 0;
}
